<?php

namespace Myriad\EventDrivenService\Models;

use Myriad\EventDrivenService\Interfaces\IUserFilterable;
use Carbon\Carbon;
use Illuminate\Http\Resources\Json\JsonResource;
use Ramsey\Uuid\Uuid;
use Illuminate\Database\Eloquent\SoftDeletes;

abstract class EventModel extends UUIDModel {

  use SoftDeletes;

  protected $commands = ['create', 'update', 'delete'];
  protected $resource = JsonResource::class;

  public final function getCommands() {
    return $this->commands;
  }

  public function format() {
    return new JsonResource($this);
  }

  public function getUpdated($lastUpdate, $perPage) {
    if (!isset($perPage)) {
      $perPage = $this->getPerPage();
    }

    $query = $this->select($this->getTable() . '.*')->withTrashed();
    if ($lastUpdate) {
      $lastUpdate = Carbon::createFromTimestamp($lastUpdate)->toDateTimeString();
      $query->where($this->getTable() . '.updated_at', '>', $lastUpdate);
    }

    if ($this instanceof IUserFilterable) {
      $query->filterForUser();
    }

    return $query->orderBy($this->getTable() . '.updated_at')->paginate($perPage);
  }

  public function createModel($event) {
    $this->cleanEvent($event);
    $this->fill($event['value'])->save();
  }

  public function updateModel($event) {
    $this->{$event['name']} = $event['value'];
    $this->{$event['name'] . '_version'} = $event['version_new'];
    $this->save();
  }

  public function deleteModel($event) {
    $this->delete();
  }

  public final function isCurrent($name, $version) {
    $currentVersion = $this[$name . '_version'];
    return $currentVersion == null || $currentVersion == $version;
  }

  protected function cleanEvent(&$event) {
    foreach ($event['value'] as $key => $value) {
      if ($key == 'uuid') {
        continue;
      }

      $column = substr($key, -8) != '_version' ? $key : substr($key, 0, strlen($key) - 8);
      $version = substr($key, -8) == '_version' ? $key : $key . '_version';

      if ((!array_key_exists($column, $event['value']) || !array_key_exists($version, $event['value'])) ||
        ($event['value'][$column] == null && $event['value'][$version] == null) ||
        !Uuid::isValid($event['value'][$version])) {
        unset($event['value'][$column]);
        unset($event['value'][$version]);
      }
    }
    return $event;
  }
}